//
// OptionCallback.h
//
// $Id: //poco/1.4/Util/include/Poco/Util/OptionCallback.h#1 $
//
// Library: Util
// Package: Options
// Module:  OptionCallback
//
// Definition of the OptionCallback class.
//
// Copyright (c) 2006, Applied Informatics Software Engineering GmbH.
// and Contributors.
//
// Permission is hereby granted, free of charge, to any person or organization
// obtaining a copy of the software and accompanying documentation covered by
// this license (the "Software") to use, reproduce, display, distribute,
// execute, and transmit the Software, and to prepare derivative works of the
// Software, and to permit third-parties to whom the Software is furnished to
// do so, all subject to the following:
// 
// The copyright notices in the Software and this entire statement, including
// the above license grant, this restriction and the following disclaimer,
// must be included in all copies of the Software, in whole or in part, and
// all derivative works of the Software, unless such copies or derivative
// works are solely in the form of machine-executable object code generated by
// a source language processor.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
// SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
// FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
//


#ifndef Util_OptionCallback_INCLUDED
#define Util_OptionCallback_INCLUDED


#include "Poco/Util/Util.h"


namespace Poco {
namespace Util {


class Util_API AbstractOptionCallback
	/// Base class for OptionCallback.
{
public:
	virtual void invoke(const std::string& name, const std::string& value) const = 0;
		/// Invokes the callback member function.
		
	virtual AbstractOptionCallback* clone() const = 0;
		/// Creates and returns a copy of the object.

	virtual ~AbstractOptionCallback();
		/// Destroys the AbstractOptionCallback.

protected:
	AbstractOptionCallback();
	AbstractOptionCallback(const AbstractOptionCallback&);
};


template <class C>
class OptionCallback: public AbstractOptionCallback
	/// This class is used as an argument to Option::callback().
	///
	/// It stores a pointer to an object and a pointer to a member
	/// function of the object's class.
{
public:
	typedef void (C::*Callback)(const std::string& name, const std::string& value);

	OptionCallback(C* pObject, Callback method):
		_pObject(pObject),
		_method(method)
		/// Creates the OptionCallback for the given object and member function.
	{
		poco_check_ptr (pObject);
	}
	
	OptionCallback(const OptionCallback& cb):
		AbstractOptionCallback(cb),
		_pObject(cb._pObject),
		_method(cb._method)
		/// Creates an OptionCallback from another one.
	{
	}
	
	~OptionCallback()
		/// Destroys the OptionCallback.
	{
	}
	
	OptionCallback& operator = (const OptionCallback& cb)
	{
		if (&cb != this)
		{
			this->_pObject = cb._pObject;
			this->_method  = cb._method;
		}
		return *this;
	}
	
	void invoke(const std::string& name, const std::string& value) const
	{
		(_pObject->*_method)(name, value);
	}
	
	AbstractOptionCallback* clone() const
	{
		return new OptionCallback(_pObject, _method);
	}
	
private:
	OptionCallback();
	
	C* _pObject;
	Callback _method;
};


} } // namespace Poco::Util


#endif // Util_OptionCallback_INCLUDED
